PRO irc_imcombine_n_reform,inimage_reform,inimage_reform_short,avgimage, $
                           rmsimage=rmsimage,comb_mode=comb_mode,weight=weight, $
                           no_shift=no_shift,dx=dx,dy=dy,follow_NIR=follow_NIR, $
                           clip_sig=clip_sig,biweight=biweight,sky_adjust=sky_adjust, $
                           reformed_inimage=reformed_inimage, $
                           reformed_short_inimage=reformed_short_inimage,IMAG=IMAG

; combine all frames within same short/long frames
;
; OPTIONS:
; simple: use total function for quick debugging.
; median_mode: median averaging will be used
; mode norm: normalization by mode value will be performed

; input: inimage[x,y,subframe_id,exposure_cycle]
;    subframe_id=0,1 for N
;    subframe_id=0,1,2,3 for M
;    exposure_cycle=0,1,2,...N
; output: avgimage[x,y,subframe_id], rmsimage[x,y,long_short_id]
;    long_short_id=0 for short
;    long_short_id=1 for long


if keyword_set(comb_mode) EQ 0 then comb_mode='average'
if keyword_set(clip_sig) EQ 0 then clip_sig=3.

;s=size(reformed_inimage) &  ncol=s[1] & nrow=s[2]
s=size(inimage_reform) &  ncol=s[1] & nrow=s[2]
if s[0] EQ 3 then n_long_frames=s[3]
;inimage_reform=reformed_inimage
;s=size(reformed_short_inimage)
s=size(inimage_reform_short)
if s[0] EQ 3 then n_short_frames=s[3]
;inimage_reform_short=reformed_short_inimage

avgimage=fltarr(ncol,nrow,2) & rmsimage=fltarr(ncol,nrow,2) ; 2 for (short, long) frames

if (keyword_set(no_shift) EQ 0) && (!IRC_SPECRED_OBSPARAM.slit_mode EQ 0B) && (comb_mode NE 'no_combine') then begin
   if keyword_set(follow_NIR) then begin
      do_image_shift,inimage_reform,      dx,dy,inimage_reform_shift,      /follow_NIR,imag=imag
      if keyword_set(IMAG) then inimage_reform_shift_short=inimage_reform_short else begin
         dx_short=dx & dy_short=dy
         do_image_shift,inimage_reform_short,dx_short,dy_short,inimage_reform_shift_short,/follow_NIR,/short
      endelse
   endif else begin
      do_image_shift,inimage_reform,      dx,dy,inimage_reform_shift,imag=imag
      if (keyword_set(IMAG)) && (!IRC_SPECRED_OBSPARAM.AOT NE '02B') || (!IRC_SPECRED_PROCESSOPTION.no_short_mode EQ 1B) then $
         inimage_reform_shift_short=inimage_reform_short $
      else begin
         dx_short=dx & dy_short=dy
         do_image_shift,inimage_reform_short,dx_short,dy_short,inimage_reform_shift_short,/short
      endelse
   endelse
endif else begin
   inimage_reform_shift      =inimage_reform
   inimage_reform_shift_short=inimage_reform_short
endelse

inimage_reform=0B               ; conserve memory
inimage_reform_shoft=0B

if keyword_set(sky_adjust) then begin
   irc_skyadjust_cube,inimage_reform_shift,      inimage_reform_shift_skyadj
   irc_skyadjust_cube,inimage_reform_shift_short,inimage_reform_shift_skyadj_short
;  inimage_reform_shift_skyadj_short=inimage_reform_shift_short
endif else begin
   inimage_reform_shift_skyadj=      inimage_reform_shift
   inimage_reform_shift_skyadj_short=inimage_reform_shift_short
endelse

inimage_reform_shift=0B; conserve memory
inimage_reform_shift_short=0B

case comb_mode of
   'no_combine': begin
      avgimage=inimage          ; simply copy the input image to output
      rmsimage=fltarr(size(inimage,/dimensions)) ; all zero rms
      return                    ; and exit.
   end

   'simple': begin
      avgimage[*,*,1]=total(inimage_reform,3)/n_long_frames
      rmsimage[*,*,1]=0.
      avgimage[*,*,0]=total(inimage_reform_short,3)/n_short_frames
      rmsimage[*,*,0]=0.
      return
   end

   'median': begin
      for x=0,ncol-1 do begin for y=0,nrow-1 do begin
         avgimage[x,y,1]=median(inimage_reform_shift_skyadj[x,y,*])
         rmsimage[x,y,1]=0.     ;robust_sigma(inimage_reform_shift_skyadj[x,y,*])
         
         avgimage[x,y,0]=median(inimage_reform_shift_skyadj_short[x,y,*])
         rmsimage[x,y,0]=0.     ;robust_sigma(inimage_reform_shift_skyadj_short[x,y,*])
      endfor & endfor
   end

   'average': begin
      n_data=n_elements(inimage_reform_shift_skyadj[0,0,*])
      for x=0,ncol-1 do begin for y=0,nrow-1 do begin
         if keyword_set(mode_nome) then use_array=where(reform(inimage_reform_shift_skyadj[x,y,*],n_data) LT 1.5, count) $
         else                           use_array=where(finite(reform(inimage_reform_shift_skyadj[x,y,*],n_data)) EQ 1,count)

         case 1 of
            (count GE 4): begin
               if keyword_set(biweight) then $
;                 meanclip,inimage_reform_shift_skyadj[x,y,use_array],avg,sigma,clipsig=3. $
                  avg=biweight_mean(inimage_reform_shift_skyadj[x,y,use_array],sigma,weight) $
               else begin
;                 meanclip,inimage_reform_shift_skyadj[x,y,use_array],avg,sigma,clipsig=clip_sig
                  meanclip_mod,inimage_reform_shift_skyadj[x,y,use_array],avg,sigma,clipsig=clip_sig,weight=weight
;                 robomea_mod,inimage_reform_shift_skyadj[x,y,use_array],clip_sig,0.5,avg,avgdev,sigma,var,skew,kurt,nfinal,new,error=error
               endelse

            end
            (count GE 2 && count LT 4): begin
               result=moment(inimage_reform_shift_skyadj[x,y,use_array],/NAN)
               avg=result[0] & sigma=sqrt(result[1])
            end
            (count EQ 1): begin
               avg=inimage_reform_shift_skyadj[x,y,use_array] & sigma=0.
            end
            else: begin
               avg=!values.f_nan & sigma=!values.f_nan
            end
         endcase
         
         avgimage[x,y,1]=avg & rmsimage[x,y,1]=sigma
         if defined(error) && error EQ 1 then begin
             avgimage[x,y,1]=!values.f_nan & rmsimage[x,y,1]=!values.f_nan
          endif

         use_array=where(inimage_reform_short[x,y,*] NE !values.f_nan,count)
         if count GE 2 then begin
            result=moment(reform(inimage_reform_short[x,y,*],n_elements(inimage_reform_short[x,y,*])))
            avgimage[x,y,0]=result[0] & rmsimage[x,y,0]=sqrt(result[1])
         endif else begin
            avgimage[x,y,0]=inimage_reform_short[x,y,*] & rmsimage[x,y,0]=0.
         endelse
      endfor & endfor

   end

endcase

;message,'Finish'

END
